#ifndef _GTL_COMPOSITE
#define _GTL_COMPOSITE

#include <list>
#include <vector>

namespace gtl
{

template <class T>
class composite_iterator
  {
  private:
    std::list<T*>* _components;
    typename std::list<T*>::const_iterator _comp_iter;
    typename T::iterator _iter;
  public:
    composite_iterator(std::list<T*>* components) : _components(components) { this->begin(); }

    composite_iterator& operator=(const composite_iterator& i)
      {
        _comp_iter = i._comp_iter;
        _iter = i._iter;
        return *this;
      }
    typedef typename T::element_type E;
    E& operator*() const { return *_iter; }
    E* operator->() const { return _iter; }
    bool operator==(const composite_iterator& i) const
      { return _comp_iter==i._comp_iter && _iter==i._iter; }
    bool operator!=(const composite_iterator& i) const
      { return _comp_iter!=i._comp_iter || _iter!=i._iter; }

    composite_iterator& operator++()
      {
        if (_comp_iter!=_components->end())
          {
            ++_iter;
            if (_iter==(*_comp_iter)->end()) 
              {
                ++_comp_iter;
                if (_comp_iter!=_components->end())
                  {
                    _iter = (*_comp_iter)->begin();
                  }
              }
          }
        return *this;
      }
    composite_iterator operator++(int)
      {
        composite_iterator i=*this;
        if (_comp_iter!=_components->end())
          {
            ++_iter;
            if (_iter==(*_comp_iter)->end()) 
              {
                ++_comp_iter;
                if (_comp_iter!=_components->end()) _iter = (*_comp_iter)->begin();
              }
          }
        return i;
      }

    composite_iterator& begin() 
      {
        _comp_iter = _components->begin();
        if (_comp_iter!=_components->end()) _iter = (*_comp_iter)->begin();
        return *this;
      }
    composite_iterator& end() 
      {
        _comp_iter = _components->end();
        if (_components->begin()!=_comp_iter)
          {
            --_comp_iter;
            _iter = (*_comp_iter)->end();
          }
        ++_comp_iter;
        return *this;
      }

  };


template <class T>
class composite
  {
  private:
    std::list<T*> _components;
  public:
    composite()
      {}
    composite operator+=(T* v)
      { _components.push_back(v); return *this; }

    typedef composite_iterator<T> iterator;
    iterator begin() { iterator i(&_components); return i.begin(); }
    iterator end() { iterator i(&_components); return i.end(); }

    friend std::ostream& operator<<(std::ostream& os, const composite& comp)
      {
        typename std::list<T*>::const_iterator i;
        typename T::iterator j;
        os << "{" ;
        i = comp._components.begin();
        for (i=comp._components.begin(); i!=comp._components.end(); ++i)
            for (j=(*i)->begin(); j!=(*i)->end(); ++j)
              {
                os << *j;
              }
        return os << "}";
      }

  };

} // namespace gtl

#endif // _GTL_COMPOSITE
