#include <qstring.h>
#include "grobj.h"
#include "vertex_array.H"
#include "line.H"
#include "circle.H"
#include "algorithm.H"
using namespace gtl;

Grobj::Grobj (grobjtype t, QCanvas * c):
type (t), canvas (c)
{
    complete = false;
    switch (type)
      {
      case linetype:
	  va = new line <> ();
	  bak_va = new line <> ();
	  last_item = new QCanvasLine (canvas);
	  ((QCanvasLine *) last_item)->setPen (QPen (Qt::red, 1));
	  break;
      case circletype:
	  va = NULL;
	  last_item = NULL;
	  break;
      };
}


Grobj::~Grobj ()
{
    delete va;
    delete bak_va;
    while (cil.empty () == FALSE)
      {
	  delete cil.back ();
	  cil.pop_back ();
      }
    delete last_item;
    last_item = NULL;
}

QString Grobj::text ()
{
    QString buf (""), v;
    vertex_array <>::iterator i;
    for (i = va->begin (); i != va->end (); i++)
      {
	  v = QString ("<vertex x='%1' y='%2'/>").arg ((double) i->x ()).arg ((double) i->y ());
	  buf.append (v);
      }
    switch (type)
      {
      case linetype:
	  buf.prepend ("<line>");
	  buf.append ("</line>");
	  break;
      case circletype:
	  buf.prepend ("<circle>");
	  buf.append ("</circle>");
	  break;
      }
    return buf;
}


void
Grobj::add (vertex < int >&v)
{
    vertex <> w = vertex <> ((long double) v.x (), (long double) v.y ());
    QCanvasRectangle *r = new QCanvasRectangle (v.x () - 2, v.y () - 2, 4, 4, canvas);
    r->show ();
    cil += r;

    switch (type)
      {
      case linetype:
	  (*va) += w;
	  (*bak_va) += w;
	  if (va->size () >= 2)
	    {
		QCanvasLine *l = new QCanvasLine (canvas);
		l->setPoints (lrintl (activepts.back ().x ()), lrintl (activepts.back ().y ()), v.x (), v.y ());
		l->show ();
		cil += l;
		complete = true;
	    }
	  activepts.push_back (w);
	  break;
      case circletype:
	  activepts.push_back (w);
	  switch (activepts.size ())
	    {
	    case 1:
		last_item = new QCanvasEllipse (canvas);
		((QCanvasEllipse *) last_item)->setBrush (QBrush (Qt::black, Qt::DiagCrossPattern));
		last_item->move (v.x (), v.y ());
		break;
	    case 2:
		va = new circle <> (activepts[0], activepts[1]);
		bak_va = new circle <> (activepts[0], activepts[1]);
		complete = true;
		int s = lrintl ((activepts[0] - w).abs ());
		((QCanvasEllipse *) last_item)->setSize (s, s);
		last_item->move ((activepts[0].x () + w.x ()) / 2, (activepts[0].y () + w.y ()) / 2);
		cil += last_item;
		last_item->show ();
		last_item = NULL;
		activepts.clear ();
		break;
	    }
	  break;
      };
}

void
Grobj::add (double x, double y)
{
    vertex <> v = vertex <> ((long double) x, (long double) y);
    activepts.push_back (v);
}

void
Grobj::finish ()
{
    switch (type)
      {
      case linetype:
	  if (activepts.size () < 2)
	      throw GrobjException ();
	  va = new line <> ();
	  bak_va = new line <> ();
	  for ( unsigned int i=0; i < activepts.size (); i++ )
	    {
	      (*va) += activepts[i];
	      (*bak_va) += activepts[i];
	    }
	  break;
      case circletype:
	  if (activepts.size () != 2)
	      throw GrobjException ();
	  va = new circle <> (activepts[0], activepts[1]);
	  bak_va = new circle <> (activepts[0], activepts[1]);
	  break;
      }
    activepts.clear ();
    recreate ();
}

void
Grobj::set_last (vertex < int >&v)
{
    vertex <> w = vertex <> ((long double) v.x (), (long double) v.y ());
    if ((NULL != last_item) && (0 < activepts.size ()))
	switch (type)
	  {
	  case linetype:
	      ((QCanvasLine *) last_item)->setPoints (lrintl (activepts.back ().x ()), lrintl (activepts.back ().y ()), v.x (), v.y ());
	      last_item->show ();
	      break;
	  case circletype:
	      int s = lrintl ((activepts.back () - w).abs ());
	      ((QCanvasEllipse *) last_item)->setSize (s, s);
	      last_item->move ((activepts[0].x () + w.x ()) / 2, (activepts[0].y () + w.y ()) / 2);
	      last_item->show ();
	      break;
	  };
}

void
Grobj::del_last ()
{
    delete last_item;
    last_item = NULL;
}

bool Grobj::is_complete ()
{
    return complete;
}

vertex < int >
Grobj::get_centre ()
{
    vertex_array <>::iterator i = va->begin ();
    centre = *i;
    ++i;
    while ( i != va->end () )
      {
	centre += *i;
	++i;
      }
    centre = centre * ((double) 1 / va->size ());
    vertex < int >ci = vertex < int >(lrintl (centre.x ()), lrintl (centre.y ()));
    return ci;
}


bool Grobj::collide (vertex < int >v)
{
    vertex <> w = vertex <> ((long double) v.x (), (long double) v.y ());
    vertex_array <>::iterator i;
    for (i = va->begin (); i < va->end (); i++)
	if ((*i - w).abs () < 3)
	    return true;
    return false;
}

void
Grobj::move (vertex < int >v)
{
    restore_va();
    vertex <> w = vertex <> ((long double) v.x (), (long double) v.y ());
    gtl::translate (*va, w);
    recreate ();
}

void
Grobj::rotate (vertex < int >from, vertex < int >to)
{
    restore_va();
    long double phi = atan2l (to.y () - centre.y (),
			      to.x () - centre.x ()) - atan2l (from.y () - centre.y (),
							       from.x () - centre.x ());
    gtl::rotate (*va, phi, centre);
    recreate ();
}

void
Grobj::scale (double s)
{
    restore_va();
    gtl::scale (*va, s, centre);
    recreate ();
}

void
Grobj::stretch (vertex < int >from, vertex < int >to)
{
    restore_va();
    vertex <> dfrom = vertex <> ((long double) from.x (), (long double) from.y ());
    vertex <> dto = vertex <> ((long double) to.x (), (long double) to.y ());
    vertex <> by = ((dto - centre).abs () / ((dfrom - centre).abs () * (dfrom - centre).abs ())) * (dfrom - centre);
    gtl::stretch (*va, by, centre);
    recreate ();
}


void
Grobj::recreate ()
{
    while (!cil.empty ())
      {
	  delete cil.back ();
	  cil.pop_back ();
      }
    QCanvasRectangle *r;
    vertex_array <>::iterator prev, i;
    switch (type)
      {
      case linetype:
	  prev = i = va->begin ();
	  r = new QCanvasRectangle (lrintl (i->x () - 2), lrintl (i->y ()) - 2, 4, 4, canvas);
	  r->show ();
	  cil += r;
	  for (i++; i < va->end (); i++)
	    {
		r = new QCanvasRectangle (lrintl (i->x ()) - 2, lrintl (i->y ()) - 2, 4, 4, canvas);
		r->show ();
		cil += r;
		QCanvasLine *l = new QCanvasLine (canvas);
		l->setPoints (lrintl (prev->x ()), lrintl (prev->y ()), lrintl (i->x ()), lrintl (i->y ()));
		l->show ();
		cil += l;
		prev = i;
	    }
	  break;
      case circletype:
	  vertex <> a, b;
	  a = *va->begin ();
	  b = *++va->begin ();
	  r = new QCanvasRectangle (lrintl (a.x ()) - 2, lrintl (a.y ()) - 2, 4, 4, canvas);
	  r->show ();
	  cil += r;
	  r = new QCanvasRectangle (lrintl (b.x ()) - 2, lrintl (b.y ()) - 2, 4, 4, canvas);
	  r->show ();
	  cil += r;
	  QCanvasEllipse *e = new QCanvasEllipse (canvas);
	  e->setBrush (QBrush (Qt::black, Qt::DiagCrossPattern));
	  e->move ((a.x () + b.x ()) / 2, (a.y () + b.y ()) / 2);
	  int s = lrintl ((a - b).abs ());
	  e->setSize (s, s);
	  e->show ();
	  cil += e;
	  break;
      };
}

void
Grobj::restore_va()
{
  if ( va->size() != bak_va->size() ) throw;
  vertex_array<>::iterator i = va->begin();
  vertex_array<>::iterator j = bak_va->begin();
  while ( i != va->end() )
    {
      *i = *j;
      ++i;
      ++j;
    }
}

void
Grobj::fix_va()
{
  if ( va->size() != bak_va->size() ) throw;
  vertex_array<>::iterator i = va->begin();
  vertex_array<>::iterator j = bak_va->begin();
  while ( i != va->end() )
    {
      *j = *i;
      ++i;
      ++j;
    }
}
