#include "mycanvasview.h"
#include <qmainwindow.h>
#include <qstatusbar.h>
#include <qstring.h>
#include <qfile.h>
#include <qxml.h>
#include "line.H"

class StructureParser:public QXmlDefaultHandler
{
  public:
    StructureParser (list < Grobj * >*pobjlist, QCanvas * pcanvas)
    {
	QXmlDefaultHandler ();
	objlist = pobjlist;
	canvas = pcanvas;
	line = QString ("line");
	circle = QString ("circle");
	vertex = QString ("vertex");
	x = QString ("x");
	y = QString ("y");
    }
    bool startDocument ()
    {
	Grobj *obj;
	while (objlist->empty () != true)
	  {
	      obj = objlist->front ();
	      objlist->pop_front ();
	      delete obj;
	  }
	act_obj = NULL;
	return TRUE;
    }
    bool startElement (const QString &, const QString &, const QString & qName, const QXmlAttributes & atts)
    {
	bool ok;
	if (qName == line)
	    act_obj = new Grobj (linetype, canvas);
	else if (qName == circle)
	    act_obj = new Grobj (circletype, canvas);
	else if (qName == vertex)
	  {
	      if (atts.value (x).isNull () == FALSE
		  && atts.value (x).isEmpty () == FALSE && atts.value (y).isNull () == FALSE && atts.value (y).isEmpty () == FALSE)
		{
		    double dx, dy;
		    dx = atts.value (x).toDouble (&ok);
		    if (ok == FALSE)
			return FALSE;
		    dy = atts.value (y).toDouble (&ok);
		    if (ok == FALSE)
			return FALSE;
		    act_obj->add (dx, dy);
		}
	      else
		  return FALSE;
	  }
	return TRUE;
    }
    bool endElement (const QString &, const QString &, const QString & qName)
    {
	if (qName == line || qName == circle)
	  {
	      act_obj->finish ();
	      objlist->push_back (act_obj);
	  }
	return TRUE;
    }

  private:
    QString x, y;
    list < Grobj * >*objlist;
    QCanvas *canvas;
    Grobj *act_obj;
    QString line, circle, vertex;
};


MyCanvasView::MyCanvasView (QCanvas * canvas, QWidget * parent, const char *name, WFlags f):
QCanvasView (canvas, parent, name, f)
{
    StructureParser *handler = new StructureParser (&objlist, canvas);
    reader.setContentHandler (handler);
    status = ((QMainWindow *) parent)->statusBar ();
    activeopn = nopn;
    activeobj = NULL;
}

MyCanvasView::~MyCanvasView ()
{
    QXmlContentHandler *handler = reader.contentHandler ();
    delete handler;
}


QString MyCanvasView::text ()
{
    QString
    buf ("");
    list < Grobj * >::iterator i;
    for (i = objlist.begin (); i != objlist.end (); i++)
	buf.append ((*i)->text ());
    buf.prepend ("<xml>");
    buf.append ("</xml>");
    return buf;
}

void
MyCanvasView::read (const QString & fileName)
{
    QFile xmlFile (fileName);
    QXmlInputSource source (&xmlFile);
    if (reader.parse (source) == FALSE)
	throw ParseException ();
    canvas ()->update ();
}

void
MyCanvasView::contentsMousePressEvent (QMouseEvent * e)
{
    int x = e->x (), y = e->y ();
    vertex < int >v (x, y);
    if (e->button () == LeftButton && 0 <= x && 600 > x && 0 <= y && 600 > y)
      {
	  switch (activeopn)
	    {
	    case moveopn:
	    case rotateopn:
	    case scaleopn:
	    case stretchopn:
		for (objit = objlist.begin (); objit != objlist.end () && false == (*objit)->collide (v); objit++);
		if (objlist.end () != objit)
		  {
		    opnpts.push_back (v);
		    c = (*objit)->get_centre ();
		  }
		break;
	    default:
		break;
	    }
      }
    canvas ()->update ();
}

void
MyCanvasView::contentsMouseReleaseEvent (QMouseEvent * e)
{
    if (e->button () == RightButton)
      {
	  activeopn = nopn;
	  status->message (tr ("Ready"));
	  if (NULL != activeobj)
	    {
		activeobj->del_last ();
		if (false == activeobj->is_complete ())
		    delete activeobj;
		activeobj = NULL;
	    }
      }
    int x = e->x (), y = e->y ();
    vertex < int >v (x, y);
    if (e->button () == LeftButton && 0 <= x && 600 > x && 0 <= y && 600 > y)
      {
	  switch (activeopn)
	    {
	    case newline:
		activeobj->add (v);
		if (false == activeinlist && true == activeobj->is_complete ())
		  {
		      objlist.push_back (activeobj);
		      activeinlist = true;
		  }
		break;
	    case newcircle:
		activeobj->add (v);
		if (true == activeobj->is_complete ())
		  {
		      objlist.push_back (activeobj);
		      activeinlist = true;
		      activeopn = nopn;
		      status->message (tr ("Ready"));
		      activeobj = NULL;
		  }
		break;
	    case moveopn:
	    case rotateopn:
	    case scaleopn:
	    case stretchopn:
		if (objlist.end () != objit)
		  {
		      activeopn = nopn;
		      status->message (tr ("Ready"));
		      (*objit)->fix_va();
		  }
		break;
	    default:
		break;
	    }
      }
    canvas ()->update ();
}

void
MyCanvasView::contentsMouseMoveEvent (QMouseEvent * e)
{
    int x = e->x (), y = e->y ();
    vertex < int >v (x, y);
    if (0 <= x and 600 > x and 0 <= y and 600 > y)
      {
	  switch (activeopn)
	    {
	    case newline:
		activeobj->set_last (v);
		break;
	    case newcircle:
		activeobj->set_last (v);
		break;
	    case moveopn:
		if (objlist.end () != objit)
		  {
		      (*objit)->move (v - opnpts[0]);
		  }
		break;
	    case rotateopn:
		if (objlist.end () != objit)
		  {
//		      (*objit)->get_centre ();
		      (*objit)->rotate (opnpts[0], v);
		  }
		break;
	    case scaleopn:
		if (objlist.end () != objit)
		  {
//		      vertex < int >c = (*objit)->get_centre ();
		      long double s = (v - c).abs () / (opnpts[0] - c).abs ();
		      (*objit)->scale (s);
		  }
		break;
	    case stretchopn:
		if (objlist.end () != objit)
		  {
//		      (*objit)->get_centre ();
		      (*objit)->stretch (opnpts[0], v);
		  }
		break;
	    default:
		break;
	    }
      }
    canvas ()->update ();
}

void
MyCanvasView::line ()
{
    status->message (tr ("Drawing line"));
    activeopn = newline;
    if (false == activeinlist)
	delete activeobj;
    else if (NULL != activeobj)
	activeobj->del_last ();
    activeinlist = false;
    activeobj = new Grobj (linetype, canvas ());
}

void
MyCanvasView::circle ()
{
    status->message (tr ("Drawing circle"));
    activeopn = newcircle;
    if (false == activeinlist)
	delete activeobj;
    else if (NULL != activeobj)
	activeobj->del_last ();
    activeinlist = false;
    activeobj = new Grobj (circletype, canvas ());
}

void
MyCanvasView::move ()
{
    status->message (tr ("Move"));
    activeopn = moveopn;
    opnpts.clear ();
    if (false == activeinlist)
	delete activeobj;
    else if (NULL != activeobj)
	activeobj->del_last ();
    activeinlist = false;
    activeobj = NULL;
    objit = objlist.end ();
}

void
MyCanvasView::rotate ()
{
    status->message (tr ("Rotate"));
    activeopn = rotateopn;
    opnpts.clear ();
    if (false == activeinlist)
	delete activeobj;
    else if (NULL != activeobj)
	activeobj->del_last ();
    activeinlist = false;
    activeobj = NULL;
    objit = objlist.end ();
}

void
MyCanvasView::scale ()
{
    status->message (tr ("Scale"));
    activeopn = scaleopn;
    opnpts.clear ();
    if (false == activeinlist)
	delete activeobj;
    else if (NULL != activeobj)
	activeobj->del_last ();
    activeinlist = false;
    activeobj = NULL;
    objit = objlist.end ();
}

void
MyCanvasView::stretch ()
{
    status->message (tr ("Stretch"));
    activeopn = stretchopn;
    opnpts.clear ();
    if (false == activeinlist)
	delete activeobj;
    else if (NULL != activeobj)
	activeobj->del_last ();
    activeinlist = false;
    activeobj = NULL;
    objit = objlist.end ();
}
