
#include "qdraw.h"

#include <qimage.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qaccel.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qwhatsthis.h>

#include "filesave.xpm"
#include "fileopen.xpm"
#include "mycanvasview.h"

Qdraw::Qdraw ():QMainWindow (0, "Qdraw", WDestructiveClose)
{
    canvas = new QCanvas (600, 600);
    e = new MyCanvasView (canvas, this, "mycanvasview");

    QPixmap
	openIcon,
	saveIcon;

    QToolBar *
	fileTools = new QToolBar (this, "File Operations");
    fileTools->setLabel (tr ("File Operations"));

    openIcon = QPixmap (fileopen);
    QToolButton *
	fileOpen = new QToolButton (openIcon, tr ("Open File"), QString::null,
				    this, SLOT (choose ()), fileTools,
				    "open file");

    saveIcon = QPixmap (filesave);
    QToolButton *
	fileSave = new QToolButton (saveIcon, tr ("Save File"), QString::null,
				    this, SLOT (save ()), fileTools,
				    "save file");


    (void) QWhatsThis::whatsThisButton (fileTools);

    QString
	fileOpenText = tr ("<p><img source=\"fileopen\"> "
			   "Click this button to open a <em>new file</em>. <br>"
			   "You can also select the <b>Open</b> command " "from the <b>File</b> menu.</p>");

    QWhatsThis::add (fileOpen, fileOpenText);

    QMimeSourceFactory::defaultFactory ()->setPixmap ("fileopen", openIcon);

    QString
	fileSaveText = tr ("<p>Click this button to save the file you "
			   "are editing. You will be prompted for a file name.\n"
			   "You can also select the <b>Save</b> command " "from the <b>File</b> menu.</p>");

    QWhatsThis::add (fileSave, fileSaveText);


    QPopupMenu *
	file = new QPopupMenu (this);
    menuBar ()->insertItem (tr ("&File"), file);


    file->insertItem (tr ("&New"), this, SLOT (newDoc ()), CTRL + Key_N);

    int
	id;
    id = file->insertItem (openIcon, tr ("&Open..."), this, SLOT (choose ()), CTRL + Key_O);
    file->setWhatsThis (id, fileOpenText);

    id = file->insertItem (saveIcon, tr ("&Save"), this, SLOT (save ()), CTRL + Key_S);
    file->setWhatsThis (id, fileSaveText);

    id = file->insertItem (tr ("Save &As..."), this, SLOT (saveAs ()));
    file->setWhatsThis (id, fileSaveText);

    file->insertSeparator ();

    file->insertItem (tr ("&Close"), this, SLOT (close ()), CTRL + Key_W);

    file->insertItem (tr ("&Quit"), qApp, SLOT (closeAllWindows ()), CTRL + Key_Q);

    menuBar ()->insertSeparator ();

    QPopupMenu *
	shape = new QPopupMenu (this);
    menuBar ()->insertItem (tr ("&Shapes"), shape);
    shape->insertItem (tr ("&Line"), e, SLOT (line ()));
    shape->insertItem (tr ("&Circle"), e, SLOT (circle ()));

    menuBar ()->insertSeparator ();

    QPopupMenu *
	operation = new QPopupMenu (this);
    menuBar ()->insertItem (tr ("&Operation"), operation);
    operation->insertItem (tr ("&Move"), e, SLOT (move ()));
    operation->insertItem (tr ("&Rotate"), e, SLOT (rotate ()));
    operation->insertItem (tr ("&Scale"), e, SLOT (scale ()));
    operation->insertItem (tr ("&Stretch"), e, SLOT (stretch ()));

    menuBar ()->insertSeparator ();

    QPopupMenu *
	help = new QPopupMenu (this);
    menuBar ()->insertItem (tr ("&Help"), help);

    help->insertItem (tr ("&About"), this, SLOT (about ()), Key_F1);
    help->insertItem (tr ("About &Qt"), this, SLOT (aboutQt ()));
    help->insertSeparator ();
    help->insertItem (tr ("What's &This"), this, SLOT (whatsThis ()), SHIFT + Key_F1);

    e->setFocus ();
    setCentralWidget (e);
    statusBar ()->message (tr ("Ready"), 2000);

    resize (600, 650);

}


Qdraw::~Qdraw ()
{
}



void
Qdraw::newDoc ()
{
    Qdraw *ed = new Qdraw;
    ed->setCaption (tr ("Qdraw"));
    ed->show ();
}

void
Qdraw::choose ()
{
    QString fn = QFileDialog::getOpenFileName (QString::null, QString::null,
					       this);
    if (!fn.isEmpty ())
	load (fn);
    else
	statusBar ()->message (tr ("Loading aborted"), 2000);
}


void
Qdraw::load (const QString & fileName)
{
    QFile f (fileName);
    if (!f.open (IO_ReadOnly))
	return;
    f.close ();
    try
    {
	setCaption (tr (fileName));
	e->read (fileName);
	statusBar ()->message (tr ("Loaded document %1").arg (fileName), 2000);
    }
    catch (...)
    {
	statusBar ()->message (tr ("Loading of document %1 failed").arg (fileName), 2000);
    }
//    e->setModified( FALSE );
}


void
Qdraw::save ()
{
    if (filename.isEmpty ())
      {
	  saveAs ();
	  return;
      }

    QString text = e->text ();
    QFile f (filename);
    if (!f.open (IO_WriteOnly))
      {
	  statusBar ()->message (tr ("Could not write to %1").arg (filename), 2000);
	  return;
      }

    QTextStream t (&f);
    t << text;
    f.close ();

//    e->setModified( FALSE );

    setCaption (filename);

    statusBar ()->message (tr ("File %1 saved").arg (filename), 2000);

}


void
Qdraw::saveAs ()
{
    QString fn = QFileDialog::getSaveFileName (QString::null, QString::null,
					       this);
    if (!fn.isEmpty ())
      {
	  filename = fn;
	  save ();
      }
    else
      {
	  statusBar ()->message (tr ("Saving aborted"), 2000);
      }
}

void
Qdraw::closeEvent ( /*QCloseEvent* ce */ )
{
/*
    if ( !e->isModified() ) {
	ce->accept();
	return;
    }

    switch( QMessageBox::information( this, tr("Qt Application Example"),
				      tr("Do you want to save the changes"
				      " to the document?"),
				      tr("Yes"), tr("No"), tr("Cancel"),
				      0, 1 ) ) {
    case 0:
	save();
	ce->accept();
	break;
    case 1:
	ce->accept();
	break;
    case 2:
    default: // just for sanity
	ce->ignore();
	break;
    }
*/
}


void
Qdraw::about ()
{
    QMessageBox::about (this, tr ("Qdraw"), tr ("This example demonstrates the use of " "the Graphical Template Library."));
}


void
Qdraw::aboutQt ()
{
    QMessageBox::aboutQt (this, tr ("Qdraw"));
}
